@@ -22,23 +22,22 @@
 #include <sys/types.h>
 
 /* GLOBALS */
-struct	wavemon_conf conf;
-int	conf_items;	/* index into array storing menu items */
-static int if_list;	/* index into array of WiFi interface names */
+struct wavemon_conf conf;
+int conf_items;			/* index into array storing menu items */
+static int if_list;		/* index into array of WiFi interface names */
 
 static void version(void)
 {
-	printf("wavemon wireless monitor %s\n", WAVEMON_VERSION);
-	printf("Distributed under the terms of the GPL >=v2.\n");
+	printf("wavemon wireless monitor %s\n", PACKAGE_VERSION);
+	printf("Distributed under the terms of the GPLv3.\n");
 }
 
 static void usage(void)
- {
+{
 	printf("Usage: wavemon [ -dhlrv ] [ -i ifname ]\n\n");
 	printf("  -d            Dump the current device status to stdout and exit\n");
 	printf("  -h            This help screen\n");
 	printf("  -i <ifname>   Use specified network interface (default: auto)\n");
-	printf("  -l            Use linear scales in favour of logarithmic ones\n");
 	printf("  -r            Generate random levels (for testing purposes)\n");
 	printf("  -v            Print version number and exit\n\n");
 }
@@ -47,210 +46,205 @@ static void getargs(int argc, char *argv[])
 {
 	int arg, tmp;
 
-	while ((arg = getopt(argc, argv, "dhi:lrv")) >= 0)
+	while ((arg = getopt(argc, argv, "dhi:rv")) >= 0)
 		switch (arg) {
-			case 'd':
-				dump_parameters();
-				exit(0);
-			case 'h':
-				usage();
-				exit(0);
-			case 'i':
-				if ((tmp = ll_scan(if_list, "S", optarg)) >= 0)
-					strncpy(conf.ifname, ll_get(if_list, tmp), sizeof(conf.ifname));
-				else
-					fatal_error("no wireless extensions found on device %s", optarg);
+		case 'd':
+			dump_parameters();
+			exit(EXIT_SUCCESS);
+		case 'h':
+			usage();
+			exit(EXIT_SUCCESS);
+		case 'i':
+			if ((tmp = ll_scan(if_list, "S", optarg)) >= 0) {
+				strncpy(conf.ifname, ll_get(if_list, tmp),
+					sizeof(conf.ifname));
 				break;
-			case 'l':
-				conf.linear = true;
-				break;
-			case 'r':
-				conf.random = true;
-				break;
-			case 'v':
-				version();
-				exit(0);
-			default:
-				/* bad argument. bad bad */
-				exit(-1);
-		}
-}
-
-static void read_cf()
-{
-	char	tmp[0x100], lv[0x20], rv[0x20], *cfname;
-	char	*lp;
-	FILE	*fd;
-	int 	lnum = 0;
-	char	found;
-	struct conf_item *ci = NULL;
-	struct passwd *pw = getpwuid(getuid());
-	int	v_int;
-	char	*conv_err;
-
-	cfname = malloc(strlen(pw->pw_dir) + strlen(CFNAME) + 3);
-	strcpy(cfname, pw->pw_dir);
-	strcat(cfname, "/"); strcat(cfname, CFNAME);
-
-	if ((fd = fopen(cfname, "r"))) {
-		while (fgets(tmp, 0x100, fd)) {
-			found = 0;
-			++lnum;
-			lp = tmp + strspn(tmp, " ");
-			if (*lp != '#' && *lp != '\n') {
-				if (strcspn(lp, " =") > sizeof(lv)) {
-					fclose(fd);
-					fatal_error("parse error in %s, line %d: identifier too long", cfname, lnum);
-				}
-				strncpy(lv, lp, strcspn(lp, " ="));
-				lv[strcspn(lp, " =")] = '\0';
-
-				ll_reset(conf_items);
-				while (!found && (ci = ll_getall(conf_items)))
-					if (ci->type != t_sep && ci->type != t_func && !strcasecmp(ci->cfname, lv))
-						found = 1;
-				if (!found) {
-					fclose(fd);
-					fatal_error("parse error in %s, line %d: unknown identifier '%s'", cfname, lnum, lv);
-				}
-
-				lp += strlen(lv);
-				lp += strspn(lp, " ");
-				if (*lp++ != '=') {
-					fclose(fd);
-					fatal_error("parse error in %s, line %d: missing '=' operator in assignment", cfname, lnum);
-				}
-				lp += strspn(lp, " ");
-
-				if (strcspn(lp, " \n") > sizeof(rv)) {
-					fclose(fd);
-					fatal_error("parse error in %s, line %d: argument too long", cfname, lnum);
-				}
-				if (*lp == '\n') {
-					fclose(fd);
-					fatal_error("parse error in %s, line %d: argument expected", cfname, lnum);
-				}
-				strncpy(rv, lp, strcspn(lp, " \n"));
-				rv[strcspn(lp, " \n")] = '\0';
-
-				switch (ci->type) {
-					case t_int:
-						v_int = strtol(rv, &conv_err, 10);
-						if (*conv_err == '\0') {
-							if (v_int > ci->max) {
-								fclose(fd);
-								fatal_error("parse error in %s, line %d: value exceeds maximum of %d",
-									    cfname, lnum, (int)ci->max);
-							} else if (v_int < ci->min) {
-								fclose(fd);
-								fatal_error("parse error in %s, line %d: value is below minimum of %d",
-									    cfname, lnum, (int)ci->min);
-							} else {
-								*ci->v.i = v_int;
-							}
-						} else {
-							fclose(fd);
-							fatal_error("parse error in %s, line %d: integer value expected, '%s' found instead",
-								    cfname, lnum, rv);
-						}
-						break;
-					case t_string:
-						if (strlen(rv) <= ci->max) {
-							strncpy(ci->v.s, rv, LISTVAL_MAX);
-						} else {
-							fclose(fd);
-							fatal_error("parse error in %s, line %d: argument too long (max %d chars)",
-								    cfname, lnum, ci->max);
-						}
-						break;
-					case t_switch:
-						if (!strcasecmp(rv, "on") || !strcasecmp(rv, "enabled")
-						    || !strcasecmp(rv, "yes") || !strcasecmp(rv, "1")) {
-							*(ci->v.b) = 1;
-						} else if (!strcasecmp(rv, "off") || !strcasecmp(rv, "disabled")
-							|| !strcasecmp(rv, "no") || !strcasecmp(rv, "0")) {
-							*(ci->v.b) = 0;
-						} else {
-							fclose(fd);
-							fatal_error("parse error in %s, line %d: boolean expected, '%s' found instead",
-								    cfname, lnum, rv);
-						}
-						break;
-					case t_list:
-						if ((v_int = ll_scan(ci->list, "S", rv)) >= 0) {
-							*ci->v.b = v_int;
-						} else {
-							fclose(fd);
-							fatal_error("parse error in %s, line %d: '%s' is not a valid argument here",
-								    cfname, lnum, rv);
-						}
-						break;
-					case t_listval:
-						if ((v_int = ll_scan(ci->list, "S", rv)) >= 0) {
-							strncpy(ci->v.s,  ll_get(ci->list, v_int), LISTVAL_MAX);
-						} else {
-							fclose(fd);
-							fatal_error("parse error in %s, line %d: '%s' is not a valid argument here",
-								    cfname, lnum, rv);
-						}
-					case t_sep:	/* These two cases are missing from the enum, they are not handled */
-					case t_func:	/* To pacify gcc -Wall, fall through here */
-						break;
-
-				}
 			}
+			err_quit("no wireless extensions found on '%s'", optarg);
+		case 'r':
+			conf.random = true;
+			break;
+		case 'v':
+			version();
+			exit(EXIT_SUCCESS);
+		default:
+			/* bad argument. bad bad */
+			exit(EXIT_FAILURE);
 		}
-		fclose(fd);
-	} else if (errno != ENOENT) {
-		fatal_error("cannot open %s: %s", cfname, strerror(errno));
-	}
 }
 
-static void write_cf()
+/* Return full path of rcfile. Allocates string which must bee free()-d. */
+static char *get_confname(void)
 {
-	struct passwd *pw = getpwuid(getuid());
+	char *full_path, *homedir = getenv("HOME");
+	struct passwd *pw;
+
+	if (homedir == NULL) {
+		pw = getpwuid(getuid());
+		if (pw == NULL)
+			err_quit("can not determine $HOME");
+		homedir = pw->pw_dir;
+	}
+	full_path = malloc(strlen(homedir) + strlen(CFNAME) + 3);
+	sprintf(full_path, "%s/%s", homedir, CFNAME);
+
+	return full_path;
+}
+
+static void read_cf(void)
+{
+	char tmp[0x100], lv[0x20], rv[0x20];
 	struct conf_item *ci = NULL;
-	char	tmp[0x100], rv[0x40], *cfname;
-	char	*lp, *cp;
-	FILE	*fd;
-	int	cfld;
-	int	add;
-	int	i;
+	FILE *fd;
+	size_t len;
+	int lnum, found, v_int;
+	char *lp, *conv_err;
+	char *cfname = get_confname();
 
-	cfname = malloc(strlen(pw->pw_dir) + strlen(CFNAME) + 3);
-	strcpy(cfname, pw->pw_dir);
-	strcat(cfname, "/"); strcat(cfname, CFNAME);
+	if (access(cfname, F_OK) != 0)
+		goto done;
 
-	cfld = ll_create();
-	if ((fd = fopen(cfname, "r"))) {
-		while (fgets(tmp, 0x100, fd))
+	fd = fopen(cfname, "r");
+	if (fd == NULL)
+		err_sys("can not read configuration file '%s'", cfname);
+
+	for (lnum = 1; fgets(tmp, sizeof(tmp), fd); lnum++) {
+
+		lp = tmp + strspn(tmp, " ");
+		if (*lp == '#' || *lp == '\n')
+			continue;
+
+		len = strcspn(lp, " =");
+		if (len > sizeof(lv))
+			err_quit("parse error in %s, line %d: identifier too long",
+				 cfname, lnum);
+		strncpy(lv, lp, len);
+		lv[len] = '\0';
+		lp += len;
+
+		ll_reset(conf_items);
+		for (found = 0; !found && (ci = ll_getall(conf_items)); )
+			found = (ci->type != t_sep && ci->type != t_func &&
+				 strcasecmp(ci->cfname, lv) == 0);
+		if (!found) {
+			err_msg("%s, line %d: ignoring unknown identifier '%s'",
+				cfname, lnum, lv);
+			continue;
+		}
+
+		lp += strspn(lp, " ");
+		if (*lp++ != '=')
+			err_quit("parse error in %s, line %d: missing '=' operator in assignment",
+				 cfname, lnum);
+		lp += strspn(lp, " ");
+
+		len = strcspn(lp, " \n");
+		if (len > sizeof(rv))
+			err_quit("parse error in %s, line %d: argument too long", cfname, lnum);
+		else if (*lp == '\n')
+			err_quit("parse error in %s, line %d: argument expected", cfname, lnum);
+		strncpy(rv, lp, len);
+		rv[len] = '\0';
+
+		switch (ci->type) {
+		case t_int:
+			v_int = strtol(rv, &conv_err, 10);
+			if (*conv_err != '\0') {
+				err_quit("parse error in %s, line %d: integer value expected, '%s' found instead",
+					 cfname, lnum, rv);
+			} else if (v_int > ci->max) {
+				err_quit("parse error in %s, line %d: value exceeds maximum of %d",
+					 cfname, lnum, (int)ci->max);
+			} else if (v_int < ci->min) {
+				err_quit("parse error in %s, line %d: value is below minimum of %d",
+					 cfname, lnum, (int)ci->min);
+			} else {
+				*ci->v.i = v_int;
+			}
+			break;
+		case t_string:
+			if (strlen(rv) > ci->max)
+				err_quit("parse error in %s, line %d: argument too long (max %d chars)",
+					 cfname, lnum, ci->max);
+			strncpy(ci->v.s, rv, LISTVAL_MAX);
+			break;
+		case t_switch:
+			if (!strcasecmp(rv, "on") || !strcasecmp(rv, "yes") ||
+			    !strcasecmp(rv, "enabled") || !strcasecmp(rv, "1")) {
+				*(ci->v.b) = 1;
+			} else if (!strcasecmp(rv, "off") || !strcasecmp(rv, "no") ||
+				   !strcasecmp(rv, "disabled") || !strcasecmp(rv, "0")) {
+				*(ci->v.b) = 0;
+			} else {
+				err_quit("parse error in %s, line %d: boolean expected, '%s' found instead",
+					 cfname, lnum, rv);
+			}
+			break;
+		case t_list:
+			v_int = ll_scan(ci->list, "S", rv);
+			if (v_int < 0)
+				err_quit("parse error in %s, line %d: '%s' is not a valid argument here",
+					 cfname, lnum, rv);
+			*ci->v.b = v_int;
+			break;
+		case t_listval:
+			v_int = ll_scan(ci->list, "S", rv);
+			if (v_int < 0)
+				err_quit("parse error in %s, line %d: '%s' is not a valid argument here",
+					 cfname, lnum, rv);
+			strncpy(ci->v.s, ll_get(ci->list, v_int), LISTVAL_MAX);
+			break;
+		case t_sep:	/* These two cases are missing from the enum, they are not handled */
+		case t_func:	/* To pacify gcc -Wall, fall through here */
+			break;
+		}
+	}
+	fclose(fd);
+done:
+	free(cfname);
+}
+
+static void write_cf(void)
+{
+	char tmp[0x100], rv[0x40];
+	struct conf_item *ci = NULL;
+	char *lp, *cp;
+	int add, i;
+	FILE *fd;
+	char *cfname = get_confname();
+	int cfld = ll_create();
+
+	if (access(cfname, F_OK) == 0) {
+		fd = fopen(cfname, "r");
+		if (fd == NULL)
+			err_sys("can not read configuration file '%s'", cfname);
+		while (fgets(tmp, sizeof(tmp), fd))
 			ll_push(cfld, "s", tmp);
 		fclose(fd);
 	}
 
-	ll_reset(conf_items);
-	while ((ci = ll_getall(conf_items))) {
+	for (ll_reset(conf_items); (ci = ll_getall(conf_items)); ) {
 		if (ci->type != t_sep && ci->type != t_func &&
 		    (!ci->dep || (ci->dep && *ci->dep))) {
 			switch (ci->type) {
-				case t_int:
-					sprintf(rv, "%d", *ci->v.i);
-					break;
-				case t_string:	/* fall through */
-				case t_listval:
-					strcpy(rv, ci->v.s);
-					break;
-				case t_switch:
-					sprintf(rv, "%s", *ci->v.b ? "on" : "off");
-					break;
-				case t_list:
-					sprintf(rv, "%s", (char *)ll_get(ci->list, *ci->v.b));
-					str_tolower(rv);
-					break;
+			case t_int:
+				sprintf(rv, "%d", *ci->v.i);
+				break;
+			case t_string:	/* fall through */
+			case t_listval:
+				strcpy(rv, ci->v.s);
+				break;
+			case t_switch:
+				sprintf(rv, "%s", *ci->v.b ? "on" : "off");
+				break;
+			case t_list:
+				sprintf(rv, "%s", (char *)ll_get(ci->list, *ci->v.b));
+				str_tolower(rv);
+				break;
 				/* Fall through, the rest are dummy statements to pacify gcc -Wall */
-				case t_sep:
-				case t_func:
-					break;
+			case t_sep:
+			case t_func:
+				break;
 			}
 
 			add = 1;
@@ -259,7 +253,7 @@ static void write_cf()
 				lp = ll_get(cfld, i);
 				cp = lp += strspn(lp, " ");
 				if (!strncasecmp(cp, ci->cfname, strcspn(cp, " ="))
-					&& strlen(ci->cfname) == strcspn(cp, " =")) {
+				    && strlen(ci->cfname) == strcspn(cp, " =")) {
 					add = 0;
 					cp += strcspn(cp, "=") + 1;
 					cp += strspn(cp, " ");
@@ -285,10 +279,11 @@ static void write_cf()
 		}
 	}
 
-	if ((fd = fopen(cfname, "w")) < 0)
-		fatal_error("cannot open %s", cfname);
-	ll_reset(cfld);
-	while ((lp = ll_getall(cfld)))
+	fd = fopen(cfname, "w");
+	if (fd == NULL)
+		err_sys("can not write to configuration file '%s'", cfname);
+
+	for (ll_reset(cfld); (lp = ll_getall(cfld)); )
 		fputs(lp, fd);
 	fclose(fd);
 
@@ -296,7 +291,7 @@ static void write_cf()
 	free(cfname);
 }
 
-static void init_conf_items()
+static void init_conf_items(void)
 {
 	struct conf_item *item;
 
@@ -337,6 +332,17 @@ static void init_conf_items()
 	item->inc	= 1;
 	ll_push(conf_items, "*", item);
 
+	item = calloc(1, sizeof(*item));
+	item->name	= strdup("Level meter smoothness");
+	item->cfname	= strdup("meter_smoothness");
+	item->type	= t_int;
+	item->v.i	= &conf.meter_decay;
+	item->min	= 0;
+	item->max	= 99;
+	item->inc	= 1;
+	item->unit	= strdup("%");
+	ll_push(conf_items, "*", item);
+
 	item = calloc(1, sizeof(*item));
 	item->name	= strdup("Dynamic info updates");
 	item->cfname	= strdup("info_updates");
@@ -413,13 +419,6 @@ static void init_conf_items()
 	item->dep	= &conf.override_bounds;
 	ll_push(conf_items, "*", item);
 
-	item = calloc(1, sizeof(*item));
-	item->name	= strdup("Linear level scales");
-	item->cfname	= strdup("linear_scale");
-	item->type	= t_switch;
-	item->v.b	= &conf.linear;
-	ll_push(conf_items, "*", item);
-
 	item = calloc(1, sizeof(*item));
 	item->name	= strdup("Random signals");
 	item->cfname	= strdup("random");
@@ -497,11 +496,12 @@ static void init_conf_items()
 	ll_push(item->list, "s", "Access points");
 	ll_push(conf_items, "*", item);
 
-	/* functions */
+	/* separator (dummy entry) */
 	item = calloc(1, sizeof(*item));
-	item->type = t_sep;
+	item->type	= t_sep;
 	ll_push(conf_items, "*", item);
 
+	/* functions */
 	item = calloc(1, sizeof(*item));
 	item->name	= strdup("Save configuration");
 	item->type	= t_func;
@@ -509,31 +509,17 @@ static void init_conf_items()
 	ll_push(conf_items, "*", item);
 }
 
-void dealloc_on_exit()
-{
-	struct conf_item *ci;
-
-	if (!conf_items)
-		return;
-
-	while ((ci = ll_getall(conf_items))) {
-		if (ci->type == t_list)
-			ll_destroy(ci->list);
-	}
-	ll_destroy(conf_items);
-}
-
-static void set_defaults()
+static void set_defaults(void)
 {
 	strncpy(conf.ifname, ll_get(if_list, 0), sizeof(conf.ifname));
 
 	conf.stat_iv		= 100;
 	conf.info_iv		= 10;
 	conf.slotsize		= 4;
+	conf.meter_decay	= 0;
 
 	conf.override_bounds	= false;
 	conf.random		= false;
-	conf.linear		= false;
 
 	conf.sig_min		= -102;
 	conf.sig_max		= 10;
@@ -541,9 +527,9 @@ static void set_defaults()
 	conf.noise_max		= 10;
 
 
-	conf.lthreshold_action	= 0;	/* Disabled, Beep, Flash, Beep+Flash */
+	conf.lthreshold_action	= TA_DISABLED;
 	conf.lthreshold		= -80;
-	conf.hthreshold_action	= 0;	/* Disabled, Beep, Flash, Beep+Flash */
+	conf.hthreshold_action	= TA_DISABLED;
 	conf.hthreshold		= -10;
 
 	conf.startup_scr	= 0;
